@@ -1,7 +1,10 @@
 from ._region import Region
+from ._boundary import RegionBoundary
 from ._templates import (
     RegionQuad,
     RegionHexahedron,
+    RegionQuadBoundary,
+    RegionHexahedronBoundary,
     RegionConstantQuad,
     RegionConstantHexahedron,
     RegionQuadraticHexahedron,
